package grafos;

//import de packages necessarias
import java.awt.*;
import java.util.*;
import java.io.*;


public class Grafo {

    protected MainApp p;
    protected int m_idGrafo = 0;
    private int n_Nos = 0;
    private int n_ramos = 0;
    protected AreaDesenho m_areaDesenho;
    protected Vector m_listaNos = new Vector();
    protected Vector m_listaRamos = new Vector();

//construtor
    public Grafo(MainApp pw, int id, int l, int a) {
       p = pw;
       m_idGrafo=id;
       // criar o canvas
       m_areaDesenho = new AreaDesenho(this, l, a);
    }

//obtem a id do grafo
    public int getIdGrafo() {
    return m_idGrafo;
    }

//insere um no', previamente criado, no grafo
    public void inserirNo(No n) {
        n_Nos++;
        m_listaNos.addElement(n);
    }

//cria um no' e insere-o no grafo
    public int inserirNo(int id, String dados,  Color cor,  int x,  int y) {
    n_Nos++;
    No auxNewNo = new No(id, dados, cor, x, y,this);
    m_listaNos.addElement(auxNewNo);
    /* Retornar o id do No que foi criado */
    return id;
    }

//apaga um no', dado o seu id
    public void apagarNo(int id) {
        int i=0;
        boolean encontrou=false;
        No auxNoEncontrado = null;
        while ((i < m_listaNos.size()) && (!encontrou)) {
           auxNoEncontrado = (No)m_listaNos.elementAt(i);
           if (auxNoEncontrado.m_idNo == id) {
                encontrou = true;
                m_listaNos.removeElementAt(i);
                n_Nos--;
           }
           i++;
        }
    }

//procura um no, dado o seu id
    public No encontrarNo(int id) {
        int i=0;
        boolean encontrou=false;
        No auxNoEncontrado = null;
        while ((i < m_listaNos.size()) && (!encontrou)) {
           auxNoEncontrado = (No)m_listaNos.elementAt(i);
           if (auxNoEncontrado.m_idNo == id) {
                encontrou = true;
           }
           i++;
        }
    return auxNoEncontrado;
    }

//insere um ramo, previamente criado, no grafo
    public void inserirRamo(Ramo r) {
     n_ramos++;
     m_listaRamos.addElement(r);
    }

//cria e insere um ramo no grafo
    public int inserirRamo(int id,Color cor,int peso,boolean seta,int idNo1, int idNo2) {
    n_ramos++;
        Ramo auxNewRamo = new Ramo(id, cor, seta, peso, encontrarNo(idNo1), encontrarNo(idNo2));
        m_listaRamos.addElement(auxNewRamo);
        /* Retornar o id do No que foi criado */
    return id;
    }

//procura um ramo, dado o seu id
    public Ramo encontrarRamo(int id) {
    int i = 0;
    boolean encontrou = false;
    Ramo auxRamoEncontrado = null;
     /* Enquanto nao chegar ao fim do vector && (and) nao encontrou o No
       desejado */
    while ( ( i < m_listaRamos.size() ) && (!encontrou) ) {
        auxRamoEncontrado = (Ramo)m_listaRamos.elementAt(i);
        if (auxRamoEncontrado.m_idRamo == id) {
            encontrou = true;
        }
        i++;
      }
      return auxRamoEncontrado;
    }

//apaga um ramo, dado o seu id
    public void apagarRamo(int id) {
     int i=0;
     boolean encontrou=false;
     Ramo auxRamoEncontrado = null;
     while ((i < m_listaRamos.size()) && (!encontrou)) {
           auxRamoEncontrado = (Ramo)m_listaRamos.elementAt(i);
           if (auxRamoEncontrado.m_idRamo == id) {
                encontrou = true;
                m_listaRamos.removeElementAt(i);
                n_ramos--;
           }
           i++;
        }
    }

//remove um no, dado o seu indice da lista de nos
    public void removeNo(int index){
      m_listaNos.removeElementAt(index);
    }

//remove um ramo, dado o seu indice da lista de ramos
    public void removeRamo(int index){
      m_listaRamos.removeElementAt(index);
    }

//obtem a area de desenho
    public AreaDesenho getAreaDesenho(){
      return m_areaDesenho;
    }

// Desenha o grafo invocando ciclicamente os 2 metodos para
//desenhar no's e ramos.
    public void desenharGrafo(Graphics grafico) {
        int i=0;
        No auxNoEncontrado = null;
        Ramo auxRamoEncontrado = null;
        //desenha os nos
        while ( i < m_listaNos.size() ) {
            auxNoEncontrado = (No)m_listaNos.elementAt(i);
            auxNoEncontrado.desenharNo(grafico);
            i++;
        }
        i=0;
    //desenha os ramos;
        while (i < m_listaRamos.size()){
            auxRamoEncontrado = (Ramo)m_listaRamos.elementAt(i);
            auxRamoEncontrado.desenharRamo(grafico);
            i++;
            }
     }

    //grava o grafo para ficheiro
    public void gravarGrafo(String nomeFich){

    try {
      FileOutputStream fw = new  FileOutputStream(nomeFich, true);
      PrintWriter pw = new PrintWriter(fw, true);

      // escreve os nos para ficheiro
      for (int i=0; i<m_listaNos.size(); i++){
        No auxNo = (No) m_listaNos.elementAt(i);
        String auxSTR = "No: "+auxNo.m_idNo+", "+auxNo.m_dadosNo+", "+auxNo.m_corNo+", "+auxNo.m_x+", "+auxNo.m_y;
        pw.println(auxSTR);
      }

      // ....e faz o equivalente para a lista de Ramos
      for (int i=0; i<m_listaRamos.size(); i++){
        Ramo auxRamo = (Ramo)m_listaRamos.elementAt(i);
        String auxSTR = "Ramo: "+auxRamo.m_idRamo+", "+auxRamo.m_peso+", "+auxRamo.m_corRamo+", "+auxRamo.m_seta;
        pw.println(auxSTR);
      }

      // fecha ficheiro
      pw.close();
      fw.close();
        }  catch (IOException e) {}

    }


}